@@ -160,23 +160,23 @@ AboutDialog::AboutDialog(wxWindow * parent)
       "<td>" + _("Project leader and primary programmer") + "</td>"
       "</tr>"
       "<tr>"
-      "<td>Joshua Haberman</td>"
-      "<td>" + _("Primary developer") + "</td>"
-      "</tr>"
-      "<tr>"
       "<td>Matt Brubeck</td>"
-      "<td>" + _("Primary developer") + "</td>"
+      "<td>" + _("Lead developer") + "</td>"
       "</tr>"
       "<tr>"
       "<td>James Crook</td>"
-      "<td>" + _("Programmer") + "</td>"
+      "<td>" + _("Lead developer") + "</td>"
       "</tr>"
       "<tr>"
       "<td>Vaughan Johnson</td>"
-      "<td>" + _("Programmer") + "</td>"
+      "<td>" + _("Lead developer") + "</td>"
       "</tr>"
       "<tr>"
       "<td>Markus Meyer</td>"
+      "<td>" + _("Lead developer") + "</td>"
+      "</tr>"
+      "<tr>"
+      "<td>Joshua Haberman</td>"
       "<td>" + _("Programmer") + "</td>"
       "</tr>"
       "<tr>"
@@ -220,7 +220,7 @@ AboutDialog::AboutDialog(wxWindow * parent)
       "Logan Lewis<br>" 
       "Tino Meinen<br>"
       "Abe Milde<br>"
-      "Monty<br>"
+      "Monty Montgomery<br>"
       "Paul Nasca<br>"
       "Jason Pepas<br>"
       "Mark Phillips<br>"
@@ -270,7 +270,7 @@ AboutDialog::AboutDialog(wxWindow * parent)
    wxButton *ok = new wxButton(this, wxID_OK,
                                _("Audacious!"),
                                wxPoint(150, 372),
-                               wxSize(100, 20));
+                               wxSize(100, -1));
    ok->SetDefault();
    ok->SetFocus();
    pBoxSizer->Add(ok, 0, wxALIGN_CENTER | wxALL, 8);
