@@ -12,6 +12,7 @@
 
 #include <wx/defs.h>
 #include <wx/app.h>
+#include <wx/docview.h>
 #include <wx/log.h>
 #include <wx/window.h>
 #include <wx/intl.h>
@@ -22,6 +23,7 @@
 #include <wx/fs_zip.h>
 #include <wx/image.h>
 
+#include <wx/file.h>
 #include <wx/filename.h>
 
 #ifdef __WXGTK__
@@ -276,9 +278,86 @@ BEGIN_EVENT_TABLE(AudacityApp, wxApp)
    EVT_MENU(globalPrefsID, AudacityApp::OnMenuPreferences)
    EVT_MENU(globalAboutID, AudacityApp::OnMenuAbout)
 #endif
-
+   // Recent file event handlers.  
+   EVT_MENU_RANGE(wxID_FILE1, wxID_FILE9, AudacityApp::OnMRUFile)
 END_EVENT_TABLE()
 
+// Backend for OnMRUFile and OnMRUProject
+bool AudacityApp::MRUOpen(wxString fileName) {
+   // Most of the checks below are copied from AudacityProject::ShowFileDialog
+   // - some rationalisation might be possible.
+   
+   AudacityProject *proj = GetActiveProject();
+   
+   if(!fileName.IsEmpty()) {
+      
+      // verify that the file exists 
+      if(wxFile::Exists(fileName)) {
+         wxFileName newFileName(fileName);
+         
+         gPrefs->Write("/DefaultOpenPath", wxPathOnly(fileName));
+         
+         // Make sure it isn't already open
+         size_t numProjects = gAudacityProjects.Count();
+         for (size_t i = 0; i < numProjects; i++) {
+            if (newFileName.SameAs(gAudacityProjects[i]->GetFileName())) {
+               wxMessageBox(wxString::Format(_("%s is already open in another window."),
+                  (const char *)newFileName.GetName()),
+                  _("Error opening project"),
+                  wxOK | wxCENTRE);
+               continue;
+            }
+         }
+         
+         // DMM: If the project is dirty, that means it's been touched at
+         // all, and it's not safe to open a new project directly in its
+         // place.  Only if the project is brand-new clean and the user
+         // hasn't done any action at all is it safe for Open to take place
+         // inside the current project.
+         //
+         // If you try to Open a new project inside the current window when
+         // there are no tracks, but there's an Undo history, etc, then
+         // bad things can happen, including data files moving to the new
+         // project directory, etc.
+         if (!proj || proj->GetDirty() || !proj->GetIsEmpty()) {
+            proj = CreateNewAudacityProject(gParentWindow);
+         }
+         // This project is clean; it's never been touched.  Therefore
+         // all relevant member variables are in their initial state,
+         // and it's okay to open a new project inside this window.
+         proj->OpenFile(fileName);
+
+         // Add file to "recent files" list.
+         proj->GetRecentFiles()->AddFileToHistory(fileName);
+         gPrefs->SetPath("/RecentFiles");
+         proj->GetRecentFiles()->Save(*gPrefs);
+         gPrefs->SetPath("..");
+      }
+      else {
+         // File doesn't exist - remove file from history
+         wxMessageBox(wxString::Format(_("%s does not exist and could not be opened.\n\nIt has been removed from the history list."), 
+                      (const char *)fileName));
+         return(false);
+      }
+   }
+   return(true);
+}
+
+void AudacityApp::OnMRUFile(wxCommandEvent& event) {
+   AudacityProject *proj = GetActiveProject();
+
+   int n = event.GetId() - wxID_FILE1;
+   wxString fileName = proj->GetRecentFiles()->GetHistoryFile(n);
+
+   bool opened = MRUOpen(fileName);
+   if(!opened) {
+      proj->GetRecentFiles()->RemoveFileFromHistory(n);
+      gPrefs->SetPath("/RecentFiles");
+      proj->GetRecentFiles()->Save(*gPrefs);
+      gPrefs->SetPath("..");
+   }
+}
+
 // The `main program' equivalent, creating the windows and returning the
 // main frame
 bool AudacityApp::OnInit()
