@@ -25,6 +25,7 @@
 #include <wx/app.h>
 #include <wx/dc.h>
 #include <wx/dcmemory.h>
+#include <wx/docview.h>
 #include <wx/intl.h>
 #include <wx/string.h>
 #include <wx/ffile.h>
@@ -143,6 +144,22 @@ void SetActiveProject(AudacityProject * project)
    wxTheApp->SetTopWindow(project);
 }
 
+AudacityDropTarget::AudacityDropTarget(AudacityProject *proj)
+   : mProject(proj)
+{
+}
+
+AudacityDropTarget::~AudacityDropTarget()
+{
+}
+
+bool AudacityDropTarget::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
+{
+   for (int i = 0; i < filenames.GetCount(); i++)
+      mProject->Import(filenames[i]);
+   return true;
+}
+
 AudacityProject *CreateNewAudacityProject(wxWindow * parentWindow)
 {
    bool bMaximized;
@@ -312,7 +329,6 @@ BEGIN_EVENT_TABLE(AudacityProject, wxFrame)
     EVT_COMMAND_SCROLL_LINEDOWN(HSBarID, AudacityProject::OnScrollRightButton)
     EVT_COMMAND_SCROLL(HSBarID, AudacityProject::OnScroll)
     EVT_COMMAND_SCROLL(VSBarID, AudacityProject::OnScroll)
-    EVT_DROP_FILES(AudacityProject::OnDropFiles)
     EVT_TIMER(AudacityProjectTimerID, AudacityProject::OnTimer)
     // Update menu method
     EVT_UPDATE_UI(1, AudacityProject::OnUpdateMenus)
@@ -345,6 +361,8 @@ AudacityProject::AudacityProject(wxWindow * parent, wxWindowID id,
 {
    mDrag = NULL;
 
+   SetDropTarget(new AudacityDropTarget(this));
+   
    // MM: DirManager is created dynamically, freed on demand via ref-counting
    // MM: We don't need to Ref() here because it start with refcount=1
    mDirManager = new DirManager();
@@ -544,11 +562,6 @@ AudacityProject::AudacityProject(wxWindow * parent, wxWindowID id,
    mImporter = new Importer;
    mImportingRaw = false;
 
-#ifdef __WXMSW__
-   // Accept drag 'n' drop files
-   DragAcceptFiles(true);
-#endif
-
    gAudacityProjects.Add(this);
 }
 
@@ -613,6 +626,9 @@ AudacityProject::~AudacityProject()
    delete mTracks;
    mTracks = NULL;
 
+   delete mRecentFiles;
+   mRecentFiles = NULL;
+
    // MM: Tell the DirManager it can now delete itself
    // if it finds it is no longer needed. If it is still
    // used (f.e. by the clipboard), it will recognize this
@@ -1713,19 +1729,6 @@ void AudacityProject::OnCloseWindow(wxCloseEvent & event)
    Destroy();
 }
 
-void AudacityProject::OnDropFiles(wxDropFilesEvent & event)
-{
-   int numFiles = event.GetNumberOfFiles();
-
-   if (numFiles > 0) {
-      wxString *files = event.GetFiles();
-
-      int i;
-      for (i = 0; i < numFiles; i++)
-         Import(files[i]);
-   }
-}
-
 // static method, can be called outside of a project
 void AudacityProject::ShowOpenDialog(AudacityProject *proj)
 {
@@ -1784,15 +1787,17 @@ void AudacityProject::ShowOpenDialog(AudacityProject *proj)
       // project directory, etc.
       if (!proj || proj->mDirty || !proj->mTracks->IsEmpty()) {
          // Open in a new window
-         AudacityProject *newProject =
-            CreateNewAudacityProject(gParentWindow);
-         newProject->OpenFile(fileName);
-      } else {
-         // This project is clean; it's never been touched.  Therefore
-         // all relevant member variables are in their initial state,
-         // and it's okay to open a new project inside this window.
-         proj->OpenFile(fileName);
+         proj = CreateNewAudacityProject(gParentWindow);
       }
+      // This project is clean; it's never been touched.  Therefore
+      // all relevant member variables are in their initial state,
+      // and it's okay to open a new project inside this window.
+      proj->OpenFile(fileName);
+
+      proj->mRecentFiles->AddFileToHistory(fileName);
+      gPrefs->SetPath("/RecentFiles");
+      proj->mRecentFiles->Save(*gPrefs);
+      gPrefs->SetPath("..");
    }
 }
 
@@ -1904,10 +1909,9 @@ void AudacityProject::OpenFile(wxString fileName)
 
 bool AudacityProject::HandleXMLTag(const char *tag, const char **attrs)
 {
-   if (strcmp(tag, "audacityproject"))
-      return false;
-
    int requiredTags = 0;
+   wxString fileVersion = "";
+   wxString audacityVersion = "";
 
    // loop through attrs, which is a null-terminated list of
    // attribute-value pairs
@@ -1918,6 +1922,16 @@ bool AudacityProject::HandleXMLTag(const char *tag, const char **attrs)
       if (!value)
          break;
 
+      if (!strcmp(attr, "version")) {
+         fileVersion = value;
+         requiredTags++;
+      }
+
+      if (!strcmp(attr, "audacityversion")) {
+         audacityVersion = value;
+         requiredTags++;
+      }
+
       if (!strcmp(attr, "projname")) {
          wxString projName = value;
          wxString projPath = wxPathOnly(mFileName);
@@ -1958,10 +1972,34 @@ bool AudacityProject::HandleXMLTag(const char *tag, const char **attrs)
       }
    } // while
 
-   if (requiredTags >= 1)
-      return true;
-   else
+   if (requiredTags < 3)
       return false;
+
+   // Specifically detect newer versions of Audacity - there are two
+   // clues - the file version, and the tag changing from
+   // "audacityproject" to "project" (but with a namespace).
+   if (fileVersion.Length() != 5 || // expecting '1.1.0', for example
+       fileVersion > AUDACITY_FILE_FORMAT_VERSION ||
+       !strcmp(tag, "project")) {
+      wxString msg;
+      msg.Printf(_("This file was saved using Audacity %s.\n"
+                   "You are using Audacity %s - you need to upgrade to\n"
+                   "a newer version to open this file."),
+                 audacityVersion.c_str(),
+                 AUDACITY_VERSION_STRING);
+      wxMessageBox(msg,
+                   _("Can't open project file"),
+                   wxOK | wxCENTRE, this);
+      return false;
+   }
+
+   // If the outer tag isn't audacityproject, then who knows what kind
+   // of file we have?
+   if (strcmp(tag, "audacityproject"))
+      return false;
+
+   // Otherwise, all tests passed.  Open the file!
+   return true;
 }
 
 XMLTagHandler *AudacityProject::HandleXMLChild(const char *tag)
@@ -2259,6 +2297,22 @@ void AudacityProject::AddImportedTracks(wxString fileName,
 
 void AudacityProject::Import(wxString fileName)
 {
+   // Special case for project files.
+   if (fileName.Length() > 4 &&
+       fileName.Right(4)==".aup") {
+      wxString msg;
+      msg.Printf(_("The file you selected is an Audacity Project file.\n\n"
+                   "Only audio files, not project files, can be imported\n"
+                   "into an existing project.  To open a project file,\n"
+                   "select Open from the File menu.\n"),
+                 _("Can't import project file"),
+                 wxOK | wxCENTRE, this);
+      wxMessageBox(msg,
+                   _("Can't import project file"),
+                   wxOK | wxCENTRE, this);
+      return;
+   }
+
    Track **newTracks;
    int numTracks;
    wxString errorMessage;
@@ -2334,7 +2388,16 @@ bool AudacityProject::SaveAs()
    mFileName = fName + ".aup";
    SetTitle(GetName());
 
-   return Save(false, true);
+   bool sucess = Save(false, true);
+
+   if (sucess) {
+      mRecentFiles->AddFileToHistory(mFileName);
+      gPrefs->SetPath("/RecentFiles");
+      mRecentFiles->Save(*gPrefs);
+      gPrefs->SetPath("..");
+   }
+
+   return(sucess);
 }
 
 //
