@@ -21,6 +21,7 @@
 #include "Audacity.h"
 
 #include <wx/defs.h>
+#include <wx/dnd.h>
 #include <wx/log.h>
 #include <wx/dragimag.h>
 #include <wx/generic/dragimgg.h>
@@ -40,6 +41,7 @@
 
 const int AudacityProjectTimerID = 5200;
 
+class wxFileHistory;
 class wxWindow;
 class wxBoxSizer;
 class wxScrollEvent;
@@ -79,6 +81,16 @@ enum PlayMode {
    loopedPlay
 };
 
+class AudacityDropTarget : public wxFileDropTarget
+{
+ public:
+   AudacityDropTarget(AudacityProject *proj);
+   virtual ~AudacityDropTarget();
+   virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);
+ private:
+   AudacityProject *mProject;
+};
+
 class AudacityProject:public wxFrame,
                       public TrackPanelListener,
                       public AStatusListener,
@@ -123,6 +135,11 @@ class AudacityProject:public wxFrame,
    bool SaveAs();
    void Clear();
 
+   wxString GetFileName() { return mFileName; }
+   bool GetDirty() { return mDirty; }
+   bool GetIsEmpty() { return mTracks->IsEmpty(); }
+   wxFileHistory *GetRecentFiles() { return mRecentFiles; }
+
 #include "Menus.h"
 
    CommandManager *GetCommandManager() { return &mCommandManager; }
@@ -137,7 +154,6 @@ class AudacityProject:public wxFrame,
    void OnUpdateMenus(wxUpdateUIEvent & event);
 
    void OnActivate(wxActivateEvent & event);
-   void OnDropFiles(wxDropFilesEvent & event);
    void OnPaint(wxPaintEvent & event);
    void OnMouseEvent(wxMouseEvent & event);
    void OnIconize(wxIconizeEvent &event);
@@ -314,6 +330,9 @@ class AudacityProject:public wxFrame,
 
    bool mIsDeleting;
 
+   // Recent File and Project History
+   wxFileHistory *mRecentFiles;
+
  public:
     DECLARE_EVENT_TABLE()
 };
