@@ -161,9 +161,9 @@ wxDialog(parent, id, title, position, size, style)
 
 	// "Don't allow clipping" checkbox
    wxCheckBox *item8b = new wxCheckBox(this, ID_CLIP_CHECKBOX,
-													_("Don't allow clipping"),
+													_("Allow clipping"),
 													wxDefaultPosition, wxDefaultSize, 0);
-   item8b->SetValue(true);
+   item8b->SetValue(false);
 
    pBoxSizer_Dialog->Add(item8b, 0, wxALIGN_CENTER | wxALL, 5);
 
@@ -249,7 +249,7 @@ bool AmplifyDialog::TransferDataFromWindow()
       ratio = pow(10.0,TrapDouble(r*10, AMP_MIN, AMP_MAX)/200.0);
    }
 
-   noclip = GetClipCheckBox()->GetValue();
+   noclip = !GetClipCheckBox()->GetValue();
 
    return TRUE;
 }
@@ -332,7 +332,7 @@ void AmplifyDialog::OnAmpSlider(wxCommandEvent & event)
    double dB2 = (GetAmpSlider()->GetValue()-1) / 10.0;
    double ratio2 = pow(10.0,TrapDouble(dB2, AMP_MIN, AMP_MAX)/20.0);
 
-   if (GetClipCheckBox()->GetValue() &&
+   if (!GetClipCheckBox()->GetValue() &&
        ratio * peak > 1.0 &&
        ratio2 * peak < 1.0)
       ratio = 1.0 / peak;
@@ -354,7 +354,7 @@ void AmplifyDialog::OnClipCheckBox(wxCommandEvent & event)
 
 void AmplifyDialog::CheckClip()
 {
-   if (GetClipCheckBox()->GetValue() == true) {
+   if (!GetClipCheckBox()->GetValue() == true) {
       GetOK()->Enable(ratio * peak <= 1.0);
    }
    else {
@@ -385,7 +385,7 @@ void AmplifyDialog::OnOk(wxCommandEvent & event)
 {
    TransferDataFromWindow();
    
-   if (GetClipCheckBox()->GetValue() == true) {
+   if (!GetClipCheckBox()->GetValue() == true) {
      if (ratio * peak > 1.0)
         ratio = 1.0 / peak;
    }
