@@ -32,13 +32,17 @@ class AudioIOPrefs:public PrefsPanel {
    
    wxCheckBox *mDuplex;
 
-   wxCheckBox *mPlaythrough; // Currently Mac OS X only
+   wxCheckBox *mPlaythrough; // Hardware playthrough, currently Mac OS X only
+   wxCheckBox *mSWPlaythrough;
 
    wxChoice *mRecChoice;
    wxChoice *mPlayChoice;
 
    wxChoice *mChannelsChoice;
 
+   wxString *mPlayNames;
+   wxString *mRecNames;
+
  public:
     DECLARE_EVENT_TABLE()
 
